'use strict'
var excelExport = require('../../../lib/scripts/excelExport.js');
var sysConsts = require('../../../lib/scripts/sysConsts.json');
var excel = require('excel-export');
var exceljs = require('exceljs');

class Export {
  constructor() {
    this.charArray = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
      'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'V', 'U', 'W', 'X', 'Y', 'Z'
    ];
  }
  init() {
    var req = this.req;
    var res = this.res;
    if (!req.body || !req.body.data || req.body.data.length == 0) {
      res.end(sysConsts.WebService_PrintExportDataIsEmpty);
      return;
    }
    var data = JSON.parse(unescape(req.body.data));
    var title = data.reportName ? data.reportName : '';

    var workbook = this.configWorkBook(new exceljs.Workbook());
    let sheet = workbook.addWorksheet(title, {
      properties: {
        tabColor: {
          argb: 'red'
        }
      }
    });
    var excelData = this.rebuildData(data, sheet);

    sheet.columns = excelData.columns;

    excelData.rows.forEach((row) => {
      sheet.addRow(row)
    });

    sheet.mergeCells('A2:' + this.excelColumnIndex(excelData.columnCount - 1) + '2');
    sheet.getRow(2).font = {
      size: 18,
      bold: true,
      color: {
        argb: 'FF000000'
      }
    };
    sheet.getCell('A2').alignment = {
      vertical: 'middle',
      horizontal: 'center'
    }

    res.writeHead(200, {
      'Content-Type': 'application/vnd.openxmlformats',
      'Content-Disposition': "attachment; filename=Export.xlsx",
    });

    return function(res) {
      workbook.xlsx.write(res)
    }
  }

  rebuildData(data) {
    var columns = [];
    var rows = [];

    //设置excel中固定的列数量
    var minCellCount = 3;
    var detailFieldsLength = data.detailFields.length;
    var columnCount = detailFieldsLength < minCellCount ? minCellCount : detailFieldsLength;
    var masterSingleRowCount = Math.floor(columnCount / minCellCount); //设置主表单行需要显示的数量（默认一个主表数据占3个单元格）

    for (var i = 0; i < columnCount; i++) {
      columns.push({
        header: '',
        key: i.toString(),
        width: 10,
      });
    }

    //设置报表名
    rows.push({
      '0': data.reportName ? data.reportName : ''
    })
    //设置表头
    var footerFields = this.appandMasterData(rows, masterSingleRowCount, minCellCount, data.masterFields, false);

    //设置明细
    var dataFields = this.appandDetailDataCaption(rows, data.detailFields); //添加表头

    var rowIndex = 1;
    if (data.detailData) {
      data.detailData.itemList.forEach((item) => { //添加数据
        var row = {};
        for (var i = 0; i < dataFields.length; i++) {
          var key = i.toString();
          var data = i == 0 ? rowIndex : item[dataFields[i]];
          row[key] = data ? data : '';
        }
        rows.push(row);
        rowIndex++;
      });
    } else {
      //从session中读取
    }

    //设置表尾
    this.appandMasterData(rows, masterSingleRowCount, minCellCount, footerFields, true);
    return {
      columns: columns,
      rows: rows,
      columnCount: columnCount - 1
    }
  }

  appandDetailDataCaption(rows, data) {
    var row = {};
    var dataFields = [];
    for (var i = 0; i < data.length; i++) {
      var item = data[i];
      var key = i.toString();
      row[key] = item.caption ? item.caption : item.name;
      dataFields.push(item.dataField);
    }
    rows.push(row);
    return dataFields;
  }

  appandMasterData(rows, masterSingleRowCount, minCellCount, masterFields, isFooter) {
    var tableBottoms = [];

    for (var i = 0; i < masterFields.length;) {
      var row = {};
      var spanCount = masterSingleRowCount;
      for (var j = 0; j < masterSingleRowCount;) {
        let masterField = masterFields[i + j];
        if (!masterField) {
          break;
        }
        if (!isFooter && masterField.isFooter) {
          tableBottoms.push(masterField);
          spanCount++;
          continue;
        }
        var tempIndex = j * minCellCount;
        var key1 = tempIndex.toString();
        var key2 = (tempIndex + 1).toString();
        var key3 = (tempIndex + 2).toString();
        row[key1] = masterField.name;
        row[key2] = masterField.value;
        row[key3] = '';
        j++;
      }
      i += spanCount;
      rows.push(row);
    }

    return tableBottoms;
  }

  //todo需要做单元测试
  excelColumnIndex(columnIndex) {
    if (columnIndex <= 0) {
      return '';
    }
    var firstIndex = Math.floor(columnIndex % 26);
    var temp = this.charArray[firstIndex];
    if (columnIndex <= firstIndex) { //查找第一个字母
      return temp;
    }
    columnIndex = columnIndex - firstIndex
    return this.Loop(columnIndex, temp);
  }

  Loop(columnIndex, value) {
    if (columnIndex < 0) {
      return value;
    }
    var index = Math.floor(columnIndex / 26);
    if (index < 26) {
      return this.charArray[index] + value;
    }

    value = this.charArray[index % 26] + value;
    return this.Loop(Math.floor(columnIndex / 26 / 26), value);
  }

  configWorkBook(workbook) {
    return workbook;
  }

  needLogin() {

  }
}

module.exports = Export;
