﻿
/*
 * aprac 主模块参数配置
 */

'use strict'
var fs = require('fs');
var path = require('path');
var utils = require('./common/utils.js');

var defaultConfig = {
  /* sessionid名称和cookie的秘钥 */
  'sessionIdKey': 'aprac.sid',
  'cookieSecret': 'apracSession_',
  'SessionResave': true,
  'SessionSaveUninitialized': true,

  /* 站点名称 （别名，标记，类似程序集名称）*/
  'webName': 'nodejs',

  /* json中间件的数据包大小 */
  'jsonLimit': '1024kb',

  /* 跨域支持的域名，默认全部 */
  "allowOrigins": "*",
  "defaultAllowHeaders": "Content-Type,Cache-Control,X-CarpaAjax,ContentType,GetSettings",
  "withCredentials": false,

  /* 平台默认log4存放路径 */
  "LogPath": "log",
  /* 默认网站路径名称 */
  "webFolder": "www",
  /* 默认APP下载页面存放父路径 */
  "appFolder": "app",
  /* 网站默认首页配置，默认：Main.gspx */
  "defaultPage": "Main.gspx",
  /* 开发环境还是正式环境的标记，默认正式环境 */
  "env": true,
  /* 开启调试log日志模块,多个模块名称用逗号分隔，默认：aprac:* */
  "debug": "aprac:*",
  /*静态文件缓存过期时间*/
  "maxAge": 60 * 60 * 24 * 365,
  /* server默认监听端口*/
  "port": 8101,
  /*session过期时间，默认20分钟*/
  "sessionTimeout": 20,
  /*sql执行超时时间（单位:秒）*/
  "commandTimeout": 20,
  /*服务器启动地址*/
  "hostName": "127.0.0.1",
  /*虚拟路径*/
  "virtualPath": "",
  /* html页面编码、资源文件载入编码、Content-Type，默认：utf-8 */
  "defaultEncoding": "utf-8",
  /* 输出已格式化或压缩的脚本到客户端，默认：false */
  "enableDebugJs": false,
  /* 全局样式文件*/
  "css": [],

  "pathMap": {
    "aprac": path.join(path.resolve(__dirname, "../"), "aprac"),
    "_Sys": path.join(path.resolve(__dirname, "../"), "aprac"),
    "Carpa.Web": path.join(path.resolve(__dirname, "../"), "aprac")
  },
  "fileMatch": /^(gif|png|jpg|js|css|ico|woff|ttf|eot|svg|swf|html|woff2)$/ig,
  "mimeTypes": {
    "css": "text/css",
    "gif": "image/gif",
    "html": "text/html",
    "ico": "image/x-icon",
    "jpeg": "image/jpeg",
    "jpg": "image/jpeg",
    "js": "text/javascript",
    "json": "application/json",
    "pdf": "application/pdf",
    "png": "image/png",
    "svg": "image/svg+xml",
    "swf": "application/x-shockwave-flash",
    "woff": "application/octet-stream",
    "woff2": "application/octet-stream",
    "ttf": "application/octet-stream",
    "tiff": "image/tiff",
    "txt": "text/plain",
    "wav": "audio/x-wav",
    "wma": "audio/x-ms-wma",
    "wmv": "video/x-ms-wmv",
    "xml": "text/xml"
  }
};

class ConfigManager {
  constructor() {

  }

  getProductConfig() {
    var configFile = path.join(process.cwd(), "config.js");
    if (!fs.existsSync(configFile)) return null;

    var config = require(configFile);
    if (typeof config != "object") {
      return null;
    }
    var apracConfig = config.apracConfig;
    if (typeof apracConfig != "object") {
      return null;
    }
    return apracConfig;
  }

  initConfig() {
    var productConfig = this.getProductConfig(); // 产品配置
    if (productConfig) {
      var productPathMap = productConfig.pathMap;
      delete productConfig.pathMap;

      utils.copyTo(productConfig, defaultConfig); // 产品覆盖系统默认配置
      utils.copyTo(productPathMap, defaultConfig.pathMap);
    }

    defaultConfig.pathNames = Object.keys(defaultConfig.pathMap);

    // allowOrigins初始化配置时就转为数组
    if (defaultConfig.allowOrigins == "*") {
      defaultConfig.allowOrigins = [];
    } else {
      defaultConfig.allowOrigins = defaultConfig.allowOrigins.split(',');
    }

    if (process.env.NODE_ENV) {
      defaultConfig.env = process.env.NODE_ENV;
    } else {
      process.env.NODE_ENV = defaultConfig.env;
    }

    if (process.env.DEBUG) {
      defaultConfig.debug = process.env.DEBUG;
    } else {
      process.env.DEBUG = defaultConfig.debug;
    }

    if (process.env.PORT) {
      defaultConfig.port = process.env.PORT;
    } else {
      process.env.PORT = defaultConfig.port;
    }

    this._initPackge(defaultConfig);

    return defaultConfig;
  }

  _initPackge(config) {
    var apracJson = require('../package.json');
    var productJson = require(path.join(process.cwd(), 'package.json'));

    config.updateTime = apracJson.updateTime;
    config.version = apracJson.version;
    config.versionTag = utils.getHashCode(apracJson.version);
    config.author = typeof apracJson.author == "string" ? apracJson.author : apracJson.author.name;

    config.productUpdateTime = productJson.updateTime;
    config.productVersion = productJson.version;
    config.productVersionTag = utils.getHashCode(productJson.version);
    config.productAuthor = productJson.author;
  }
}

module.exports.apracConfig = new ConfigManager().initConfig();

/* 这个对象会全部输出到客户端，因此不能加入关键数据 */
module.exports.apracSettings = {
  "AllPrintUseService": true, // 默认使用服务打印打印（本地打印）
  "AllUseLocalPrint": false,// 使用本地打印全局开关
  "CloudPrintAppName": "",// 云打印AppName：用于本地区分样式
  "DefaultWebRequestTimeout": 0,
  "UsePrintButton": false,
  "DecimalPrecision": 14,
  "DecimalScale": 4,
  "MinClientHeight": 555,
  "MinClientWidth": 1024,
  "DecimalIntDigits": 0,
  "GridRowNoColumnCaption": "<div class='RowNoConfigIcon'></div>",
  "GridPagerDefaultShowRefreshButton": false,
  "GridPagerDefaultShowGotoButton": false,
  "GridPagerDefaultMaxPageSize": 100,
  "GridPagerDefaultPageSize":10,
  "GridDefaultRowCount": 1,
  "GridDefaultAllowCopy": true,
  "GridDefaultAllowFilter": true,
  "GridDefaultAllowPopup": false,
  "UseCustomScrollBar": false,
  "GridDefaultAllowConfig": true,
  "GridDefaultAllowFrozen": true,
  "GridDefaultAllowResize": true,
  "GridDefaultShowRowIndicator": true,
  "GridDefaultAutoPagerPageSize": false,
  "GridMultiSelectColumnDefaultShowHeaderCheckBox": true,
  "GridDefaultRowHeight": 30,
  "ButtonAutoWidth": true,
  "OkButtonText": "确定",
  "CloseButtonText": "关闭",
  "CancelButtonText": "取消",
  "DisplayThousandSeperator": false,
  "NumberEditDefaultShowCalculator": true,
  "EditFilterChars": null,
  "GotoPageButtonText": null,
  "EnableDebugJson": false,
  "RequiredCPrintVersion": "10.2.0.442",
  "CPrintDataDirName": null,
  "EnableSaveMDIPageLocation": true,
  "EnableSaveClientLog": true,
  "EnableGlobalTitle": true,
  "GridUseColumnsDisplayOrder": true,
  "CPrintExecuteNotUseIEDialog": false,
  "CPrintExecuteDirectPreview": false,
  "CPrintUseClound": false,
  "MaxMDIPageCount": 50,
  "DbPagerDataSourceDefaultUseMemorySort": true,
  "DbPagerDataSourceDefaultFildNameIgnoreCase": true,
  "ClientWidthContextName": "DefaultWidth",
  "ClientHeightContextName": "DefaultHeight",
  "UseCarpaReport": false,
  "OnlyUseCarpaReport": false,
  "UseReportDataFromCarpaServer": false,
  "DbPagerDataSourceRoundValueForSummary": false,
  "HtmlEditorEnableUploadImage": true,
  "DefaultGridSpace": 2,
  "PrintProductName": null,
  "OnlineUserManager": true,
  "GspxResolveCache": true,
  "GridDefaultSavedQueryParams": false,
  "GridPagerDropDownNumericButton": true,
  "TabButtonNormalShowCloseButton": false,
  "DefaultReadonlySession": false,
  "DateYearMinValue": 1900,
  "GridDefaultWordWrap": false,
  "GridDefaultShowLines": true
}
