﻿/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-06-06
 * 缓存客户端访问WebMethod方法的WebMethodData信息
 */
'use strict'

var path = require('path');
var fs = require('fs');
var StringBuilder = require('../../common/stringBuilder.js');
var utils = require('../../common/utils.js');
var debug = require('debug')('aprac:WebMethodData');
var sysConsts = require('../../scripts/sysConsts.json');
var config = require('../../config.js').apracConfig;

var serviceDataCache = {};
exports.getWebServiceData = function webService$getWebServiceData(req) {
  var filedir = req.parsed.filedir;
  var filename = filedir.replace('.ajax', '.jsx');

  var data = serviceDataCache[filename];
  if (data) return data;

  var error = "";
  var serviceType = null;
  var interceptor = null;
  try {
    var controller = require(filename);
    serviceType = controller.WebMethod;
    interceptor = controller.Interceptor;
    serviceType.__owner = controller;
  } catch (ex) {
    error = ex.message;
  }

  if (serviceType != null) {
    var pos = req.parsed.pathname.indexOf(".");
    var fullPath = req.parsed.pathname.substr(0, pos);
    var namespace = utils.pathToArray(fullPath).join('.');

    data = new WebMethodData(namespace, serviceType, interceptor);

    if (config.env) { // 产品发布环境才缓存
      serviceDataCache[filename] = data;
    }
  }

  if (!data) {
    throw new Error(utils.format(sysConsts.WebService_NoneWebService, req.url, error));
  }

  return data;
}

class WebMethodData {
  constructor(namespace, serviceType, interceptor) {
    this.serviceType = serviceType;
    this.interceptor = interceptor;
    this.wnamespace = namespace;
    this.namepath = namespace.replace(/\./g, "/");
    this.owner = serviceType.__owner;

    var noneLoginFun = this.serviceType.prototype.noneLogin;
    if (noneLoginFun) {
      this.noneLogins = noneLoginFun(); // ["login1","login2"]
    } else {
      this.noneLogins = [];
    }

    this.init();
  }

  init() {
    this._methods = {};
    var methodsNames = this.getNames();
    for (var i = 0, count = methodsNames.length; i < count; i++) {
      var name = methodsNames[i];
      var data = this._initMethodData(name);
      this._methods[name] = data;
    }
  }

  _initMethodData(name) {
    var data = {};
    data.name = name;
    data.type = this.serviceType;
    data.args = utils.getMethodArgs(this.serviceType.prototype[name]);
    data.needLogin = this.getMethodNeedLogin(name);
    return data;
  }

  getMethodNeedLogin(name) {
    var ownerLogin = this.owner.prototype && typeof (this.owner.prototype.needLogin) == "function";
    var methodNeedLogin = !this.noneLogins.includes(name);
    return ownerLogin && methodNeedLogin;
  }

  getMethodData(name) {
    return this._methods[name];
  }

  getInterceptor() {
    return this.interceptor;
  }

  getNames() {
    if (this._methodNames) return this._methodNames;

    var names = [];
    this._makeMethod(names, this.serviceType.prototype);

    this._methodNames = names;
    var index = this._methodNames.indexOf("noneLogin");
    if (index > -1) {
      this._methodNames.splice(index, 1);
    }
    return this._methodNames;
  }

  _makeMethod(names, proto) {
    var list = Object.getOwnPropertyNames(proto);
    list.shift(); // constructor

    names.push.apply(names, list);

    if (proto.__proto__ && proto.__proto__.constructor.name != 'Object') {
      this._makeMethod(names, proto.__proto__);
    }
  }

  getCount() {
    return this.getNames().length;
  }

  getMethodArgs(methodName) {
    return this._methods[methodName].args;
  }
}