﻿var log4js = require('log4js');
var fs = require("fs");
var path = require("path");
var config = require('../config.js');

class log4Helper {
    constructor(customBaseDir, openLog, projectName) {
        //默认开启日志
        if (openLog === false) {
            this.openLog = openLog;
        } else {
            this.openLog = true;
        }

        if (projectName == null) {
            projectName = "apracjs";
        } else {
            projectName = projectName.toLowerCase();
        }

        //[indexname=c_bfune_deploy_d4hist] %date{yyyy-MM-dd HH:mm:ss,fff} [%thread] %level - %message%newline%exception%newline

        //加载配置文件
        var objConfig = {
            "customBaseDir": "\\log",
            "customDefaultAtt": {
                "type": "dateFile",
                "absolute": true,
                "alwaysIncludePattern": false
            },
            "appenders": [{
                    "type": "console",
                    "category": "console",
                    "layout": {
                        "type": "pattern",
                        "pattern": "[indexname=" + projectName + "] %d{yyyy-MM-dd hh:mm:ss,fff} [%z] %p - %m%n%m%n"
                    }
                },
                {
                    "filename": 'debug/debug.log',
                    "pattern": ".yyyy-MM-dd.log",
                    "category": "logDebug",
                    "layout": {
                        "type": "pattern",
                        "pattern": "[indexname=" + projectName + "] %d{yyyy-MM-dd hh:mm:ss,000} [%z] %p - %m%nlogDebug%n"
                    }
                },
                {
                    "filename": 'info/info.log',
                    "pattern": ".yyyy-MM-dd.log",
                    "category": "logInfo",
                    "layout": {
                        "type": "pattern",
                        "pattern": "[indexname=" + projectName + "] %d{yyyy-MM-dd hh:mm:ss,000} [%z] %p - %m%nlogInfo%n"
                    }
                },
                {
                    "filename": 'warn/warn.log',
                    "pattern": ".yyyy-MM-dd.log",
                    "category": "logWarn",
                    "layout": {
                        "type": "pattern",
                        "pattern": "[indexname=" + projectName + "] %d{yyyy-MM-dd hh:mm:ss,000} [%z] %p - %m%nlogWarn%n"
                    }
                },
                {
                    "filename": 'err/err.log',
                    "pattern": ".yyyy-MM-dd.log",
                    "category": "logErr",
                    "layout": {
                        "type": "pattern",
                        "pattern": "[indexname=" + projectName + "] %d{yyyy-MM-dd hh:mm:ss,000} [%z] %p - %m%nlogErr%n"
                    }
                }
            ],
            "replaceConsole": true,
            "levels": {
                "logDebug": "DEBUG",
                "logInfo": "DEBUG",
                "logWarn": "DEBUG",
                "logErr": "DEBUG"
            }
        };

        // 检查配置文件所需的目录是否存在，不存在时创建
        if (objConfig.appenders) {
            var baseDir = customBaseDir || objConfig["customBaseDir"];

            var pathInfo = this.checkAbsPath(baseDir);
            var isAbsPath = pathInfo.isAbsPath;
            this.baseDir = baseDir = pathInfo.baseDir;
            if (!isAbsPath) {
                this.baseDir = baseDir = process.cwd() + path.sep + baseDir;
            }

            var defaultAtt = objConfig["customDefaultAtt"];

            for (var i = 0, j = objConfig.appenders.length; i < j; i++) {
                var item = objConfig.appenders[i];
                if (item["type"] == "console")
                    continue;

                if (defaultAtt != null) {
                    for (var att in defaultAtt) {
                        if (item[att] == null)
                            item[att] = defaultAtt[att];
                    }
                }
                if (baseDir != null) {
                    if (item["filename"] == null)
                        item["filename"] = baseDir;
                    else
                        item["filename"] = baseDir + item["filename"];
                }
                var fileName = item["filename"];
                if (fileName == null)
                    continue;
                var pattern = item["pattern"];
                if (pattern != null) {
                    fileName += pattern;
                }
                var category = item["category"];
                if (!this.checkAbsPath(fileName).isAbsPath) //path.isAbsolute(fileName))
                    throw new Error("配置节" + category + "的路径不是绝对路径:" + fileName);
                var dir = path.dirname(fileName);
                this.checkAndCreateDir(dir);
            }
        }

        // 目录创建完毕，才加载配置，不然会出异常
        log4js.configure(objConfig);

        this.logDebug = log4js.getLogger('logDebug');
        this.logInfo = log4js.getLogger('logInfo');
        this.logWarn = log4js.getLogger('logWarn');
        this.logErr = log4js.getLogger('logErr');
    }

    get baseDir() {
        return this._baseDir;
    }

    set baseDir(value) {
        this._baseDir = value;
    }

    get openLog() {
        return this._openLog;
    }

    set openLog(value) {
        this._openLog = value;
    }

    get logDebug() {
        return this._logDebug;
    }

    set logDebug(value) {
        this._logDebug = value;
    }

    get logInfo() {
        return this._logInfo;
    }

    set logInfo(value) {
        this._logInfo = value;
    }

    get logWarn() {
        return this._logWarn;
    }

    set logWarn(value) {
        this._logWarn = value;
    }

    get logErr() {
        return this._logErr;
    }

    set logErr(value) {
        this._logErr = value;
    }

    // 判断日志目录是否存在，不存在时创建日志目录
    checkAndCreateDir(dir) {
        if (!fs.existsSync(dir)) {
            this.mkdirsSync(dir);
        }
    }

    //创建多层文件夹 同步
    mkdirsSync(dirpath) {
        if (!fs.existsSync(dirpath)) {
            var pathtmp;
            dirpath.split(path.sep).forEach(function(dirname) {
                if (pathtmp) {
                    pathtmp = path.join(pathtmp, dirname);
                } else {
                    pathtmp = dirname;
                    pathtmp = (process.platform == "win32" ? pathtmp : "/" + pathtmp);
                }

                if (pathtmp != "" && !fs.existsSync(pathtmp)) {
                    if (!fs.mkdirSync(pathtmp)) {
                        return false;
                    }
                }
            });
        }
        return true;
    }

    writeDebug(msg, cb) {
        if (this.openLog) {
            this.logDebug.debug(this.formatLog(msg));
            if (typeof(cb) === 'function') {
                cb(msg);
            }
        }
    }

    writeInfo(msg, cb) {
        if (this.openLog) {
            this.logInfo.info(this.formatLog(msg));
            if (typeof(cb) === 'function') {
                cb(msg);
            }
        }
    }

    writeWarn(msg, cb) {
        if (this.openLog) {
            this.logWarn.warn(this.formatLog(msg));
            if (typeof(cb) === 'function') {
                cb(msg);
            }
        }
    }

    writeErr(msg, cb) {
        if (this.openLog) {
            this.logErr.error(this.formatLog(msg));
            if (typeof(cb) === 'function') {
                cb(msg);
            }
        }
    }

    getTimeStr(time) {
        var timeStr = time.getFullYear() + "-" + (time.getMonth() + 1) + "-" + time.getDate();
        return timeStr;
    }

    formatLog(msg) {
        if (msg === undefined || msg === null)
            msg = "";
        var typeName = typeof(msg);

        switch (typeName) {
            case "string":
                break;
            case "object":
                msg = JSON.stringify(msg);
                break;
            default:
                msg = "";
                break;
        }
        return msg;
    }

    checkAbsPath(baseDir) {
        if (baseDir === undefined || baseDir === null)
            throw new Error("路径不能为空");

        var isAbsPath = false; //是否是绝对路径

        if (path.sep === '\\') { //Windows
            if (baseDir[0] === '\\' || baseDir[0] === '/') {
                throw new Error("Windows系统下配置文件中日志路径不允许以正反斜杠开始！");
            }

            if (baseDir.length > 1 && baseDir[1] === ':') {
                isAbsPath = true;
            }
        } else { //Unix、MacOS、Linux等
            if (baseDir[0] === '\\') {
                throw new Error("Unix、MacOS、Linux等系统下配置文件中日志路径不允许以反斜杠开始！");
            }

            if (baseDir[0] === '/') {
                isAbsPath = true;
            }
        }

        baseDir = baseDir.replace(/\\/g, path.sep).replace(/\//g, path.sep);
        if (baseDir[baseDir.length - 1] != path.sep) {
            baseDir += path.sep;
        }

        return {
            isAbsPath: isAbsPath,
            baseDir: baseDir
        };
    }
}
exports = module.exports = log4Helper;

exports.getInstance = function() {
    if (this._instance) return this._instance;
    return this._instance = new log4Helper(config.apracConfig.LogPath, config.apracConfig.OpenLog, config.apracConfig.webName);
}